#ifndef SONY_FPGA_XPT_H
#define SONY_FPGA_XPT_H

#include <linux/types.h>
#include "sony_fpga.h"
#include "sony_fpga_cmd.h"

enum {
    RETIMER_UP,
    RETIMER_DOWN
};
// I2Cスレーブアドレス
#define I2C_SLAVE_ADDR_IOEXPANDER1     0x21
#define I2C_SLAVE_ADDR_IOEXPANDER2     0x22
#define I2C_SLAVE_ADDR_IOEXPANDER3     0x23
#define I2C_SLAVE_ADDR_QSFP            0x50

#define I2C_SLAVE_ADDR_RTM0            0x18
#define I2C_SLAVE_ADDR_RTM1            0x19
#define I2C_SLAVE_ADDR_RTM2            0x1a
#define I2C_SLAVE_ADDR_RTM3            0x1b
#define I2C_SLAVE_ADDR_RTM4            0x1c
#define I2C_SLAVE_ADDR_RTM5            0x1d
#define I2C_SLAVE_ADDR_RTM6            0x1e
#define I2C_SLAVE_ADDR_RTM7            0x1f
#define I2C_SLAVE_ADDR_RTM8            0x20
#define I2C_SLAVE_ADDR_RTM9            0x25
#define I2C_SLAVE_ADDR_RTM10           0x22
#define I2C_SLAVE_ADDR_RTM11           0x23

// I2C Driver Address
#define QSFP_1_BASE_ADDR               0x00060000
#define QSFP_2_BASE_ADDR               0x00061000
#define QSFP_3_BASE_ADDR               0x00062000
#define QSFP_CTRL1_BASE_ADDR           0x00064000
#define QSFP_CTRL2_BASE_ADDR           0x00065000
#define QSFP_CTRL3_BASE_ADDR           0x00066000
#define RTM_BASE_ADDR                  0x00063000
#define EXT_MOD_PRS_L_BASE_ADDR        0x00067020

// I2Cドライバレジスタマップ
#define FPGA_OFFSET_I2C_CMD     0x10
#define FPGA_OFFSET_I2C_PRSCL   0x14
#define FPGA_OFFSET_I2C_TX      0x20
#define FPGA_OFFSET_I2C_STS     0x38
#define FPGA_OFFSET_I2C_RX      0x3C

// I2Cコマンド
#define FPGA_VAL_I2C_CMD_TXPURGE    0x0008
#define FPGA_VAL_I2C_CMD_RXPURGE    0x0004
#define FPGA_VAL_I2C_CMD_PURGE \
    (FPGA_VAL_I2C_CMD_TXPURGE | \
     FPGA_VAL_I2C_CMD_RXPURGE)
#define FPGA_VAL_I2C_CMD_CORERST    0x0002
#define FPGA_VAL_I2C_CMD_COREEN     0x0001

// I2Cステータス
#define FPGA_VAL_I2C_STS_BUSY       0x0800
#define FPGA_VAL_I2C_STS_AL         0x0400
#define FPGA_VAL_I2C_STS_RXACK      0x0200
#define FPGA_VAL_I2C_STS_RDWR       0x0100
#define FPGA_VAL_I2C_STS_TXFULL     0x0010
#define FPGA_VAL_I2C_STS_TXEMPTY    0x0008
#define FPGA_VAL_I2C_STS_RXFULL     0x0004
#define FPGA_VAL_I2C_STS_RXHIGH     0x0002
#define FPGA_VAL_I2C_STS_RXEXIST    0x0001

#define FPGA_VAL_I2C_TX_CMD_START   0x0800
#define FPGA_VAL_I2C_TX_CMD_STOP    0x0400
#define FPGA_VAL_I2C_TX_CMD_READ    0x0200
#define FPGA_VAL_I2C_TX_CMD_WRITE   0x0100

#define I2C_PAGE_SIZE   32
#define I2C_RETRY_COUNT 200000
#define I2C_SYS_FREQ    (100*1000*1000) // 100MHz

#define I2C_PR_ERR(info, x, ...) \
    pr_err("I2C 0x%08llx: " x ", status=%x\n", \
          (u64 __force)info, ##__VA_ARGS__, i2c_get_status(info))

#define QSFP_SUPPORT_VENDOR_FINISAR "FINISAR "
#define QSFP_SUPPORT_VENDOR_CIG "CIG "

#define XPT_BOARD_REV11 0
#define XPT_BOARD_REV12 1
#define XPT_BOARD_REV13 2
#define XPT_BOARD_REV21 3

struct i2c_dev_info {
    u32 slave_addr;
    u32 addr_size;
};

struct i2c_settings {
    u32 pci_address;
    u32 slave_address;
    u32 i2c_address;
    u32 setting;
    u32 mask;
};

extern int sony_fpga_xpt_fpga_probe(struct sony_fpga_ctrl* ctrl);
extern int sony_fpga_xpt_i2c_request(struct sony_fpga_ctrl* ctrl, struct i2c_param* param, bool is_read);
extern int sony_fpga_pause_qsfp_thread(struct sony_fpga_ctrl* ctrl, u32 control);
extern int sony_fpga_resume_qsfp_thread(struct sony_fpga_ctrl* ctrl, u32 control);
extern int sony_fpga_xpt_qsfp_setting(struct sony_fpga_ctrl* ctrl, u32 qsfp_number);
extern int sony_fpga_xpt_qsfp_cdr_reset(struct sony_fpga_ctrl* ctrl, u32 qsfp_number);

extern const u32 qsfp_number_resolve[][2];
#endif
